TERMINAL = "terminal"
GAMMA = "gamma"

NAME_NOT_FOUND = "Name not found: %s"
SORT_CONFLICT = "It doesn't make sense"
WHICH_ONE = "Which one?"
UNKNOWN_WORD = "Unknown word: %s"
NO_SENTENCE = "No sentence given"
NOT_UNDERSTOOD = "Sorry, I don't understand"
NO_RESULTS = "No results"

POS_TYPE_RELATION = "relation"
POS_TYPE_WORD_FORM = "word-form"
POS_TYPE_REG_EXP = "reg-exp"

CATEGORY_TEXT = "text"
CATEGORY_PROPER_NOUN = "proper_noun"
CATEGORY_PROPER_NOUN_GROUP = "proper_noun_group"
