from setuptools import setup, find_packages

setup(
    name='richard',
    version='0.2.0',    
    description='A Natural Language Understanding and Execution library',
    url='https://github.com/garfix/richard',
    author='Patrick van Bergen',
    author_email='patrick.vanbergen@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=[],

    classifiers=[
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',  
        'Programming Language :: Python :: 3.10',
    ],
)
